<?php

/**
 * @file
 * CDEK API admin pages.
 */

/**
 * CDEK API basic settings form.
 *
 * @return array
 *   Drupal form array.
 *
 * @see cdek_api_settings_form_validate()
 */
function cdek_api_settings_form() {
  $form = array();

  // Base path for integration API.
  $form['cdek_api_integration_base_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Integration API base path'),
    '#default_value' => variable_get('cdek_api_integration_base_path', 'https://integration.cdek.ru'),
    '#size' => 60,
    '#required' => TRUE,
  );
  // Base path for API.
  $form['cdek_api_base_path'] = array(
    '#type' => 'textfield',
    '#title' => t('API base path'),
    '#default_value' => variable_get('cdek_api_base_path', 'https://api.cdek.ru'),
    '#size' => 60,
    '#required' => TRUE,
  );

  // Login for API requests with authorization.
  $form['cdek_api_auth_login'] = array(
    '#type' => 'textfield',
    '#title' => t('Login for API requests (Account)'),
    '#default_value' => variable_get('cdek_api_auth_login', NULL),
    '#size' => 60,
  );
  // Password for API requests with authorization.
  $form['cdek_api_secure_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Password for API requests (Secure password)'),
    '#default_value' => variable_get('cdek_api_secure_password', NULL),
    '#size' => 60,
  );

  // Sender city ID.
  $form['cdek_api_sender_city_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Default sender city ID'),
    '#description' => t('Integer, origin city code in accordance with city codes provided by CDEK.'),
    '#default_value' => variable_get('cdek_api_sender_city_id', NULL),
    '#size' => 30,
  );
  // Sender city postcode.
  $form['cdek_api_sender_city_postcode'] = array(
    '#type' => 'textfield',
    '#title' => t('Default sender city postcode'),
    '#description' => t('Integer, 6 symbols, zip code of the city of origin.'),
    '#default_value' => variable_get('cdek_api_sender_city_postcode', NULL),
    '#size' => 30,
  );

  // Tariff ID value.
  $form['cdek_api_tariff_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Default tariff ID'),
    '#description' => t('Integer, code of chosen tariff. Should be chosen from the list provided by CDEK.'),
    '#default_value' => variable_get('cdek_api_tariff_id', NULL),
    '#size' => 30,
  );

  // Settings validate callback.
  $form['#validate'] = array('cdek_api_settings_form_validate');
  return system_settings_form($form);
}

/**
 * CDEK API base settings form validate callback.
 *
 * @param array $form
 *   Drupal form array.
 * @param array $form_state
 *   Drupal form state array.
 *
 * @see cdek_api_settings_form()
 */
function cdek_api_settings_form_validate($form, &$form_state) {
  // Get form values.
  $values = &$form_state['values'];
  // Validate integration base path.
  if (!empty($values['cdek_api_integration_base_path'])) {
    $values['cdek_api_integration_base_path'] = trim($values['cdek_api_integration_base_path']);
    if (!valid_url($values['cdek_api_integration_base_path'], TRUE)) {
      form_set_error('cdek_api_integration_base_path', t('Integration API base path must be a valid URL.'));
    }
  }
  // Validate base path.
  if (!empty($values['cdek_api_base_path'])) {
    $values['cdek_api_base_path'] = trim($values['cdek_api_base_path']);
    if (!valid_url($values['cdek_api_base_path'], TRUE)) {
      form_set_error('cdek_api_base_path', t('API base path must be a valid URL.'));
    }
  }
  // Trim auth values.
  if (!empty($values['cdek_api_auth_login'])) {
    $values['cdek_api_auth_login'] = trim($values['cdek_api_auth_login']);
  }
  if (!empty($values['cdek_api_secure_password'])) {
    $values['cdek_api_secure_password'] = trim($values['cdek_api_secure_password']);
  }
  // Validate sender city.
  // TODO: Add validation in accordance with CDEK city ids and postcodes.
  if (!empty($values['cdek_api_sender_city_id'])) {
    $values['cdek_api_sender_city_id'] = trim($values['cdek_api_sender_city_id']);
    // Check integer format.
    if (!ctype_digit($values['cdek_api_sender_city_id'])) {
      form_set_error('cdek_api_sender_city_id', t('Sender city ID must contain only digits.'));
    }
  }
  if (!empty($values['cdek_api_sender_city_postcode'])) {
    $values['cdek_api_sender_city_postcode'] = trim($values['cdek_api_sender_city_postcode']);
    // Check integer format.
    if (!ctype_digit($values['cdek_api_sender_city_postcode'])) {
      form_set_error('cdek_api_sender_city_postcode', t('Sender city postcode must contain only digits.'));
    }
  }
  // Validate tariff ID.
  if (!empty($values['cdek_api_tariff_id'])) {
    $values['cdek_api_tariff_id'] = trim($values['cdek_api_tariff_id']);
    // Check integer format.
    if (!ctype_digit($values['cdek_api_tariff_id'])) {
      form_set_error('cdek_api_tariff_id', t('Tariff ID must contain only digits.'));
    }
  }
}

/**
 * CDEK API advanced settings form.
 *
 * @return array
 *   Drupal form array.
 *
 * @see cdek_api_advanced_settings_form_validate()
 * @see cdek_api_advanced_settings_form_submit()
 */
function cdek_api_advanced_settings_form() {
  $form = array();

  // API request timeout.
  $form['cdek_api_request_timeout'] = array(
    '#type' => 'textfield',
    '#title' => t('API request timeout'),
    '#description' => t('Float representing the maximum number of seconds the API request may take. The default is 5 seconds.'),
    '#default_value' => variable_get('cdek_api_request_timeout', 5),
    '#size' => 5,
    '#required' => TRUE,
  );

  // API data cache expire.
  $form['cdek_api_cache_expire'] = array(
    '#type' => 'textfield',
    '#title' => t('API data cache expire'),
    '#description' => t('API data cache expire in seconds. Set @permanent for permanent cache, set @temporary for temporary cache, leave empty for non-caching.', array(
      '@permanent' => CACHE_PERMANENT,
      '@temporary' => CACHE_TEMPORARY,
    )),
    '#default_value' => variable_get('cdek_api_cache_expire', NULL),
    '#size' => 15,
  );

  // Settings validate callback.
  $form['#validate'] = array('cdek_api_advanced_settings_form_validate');
  $form['#submit'] = array('cdek_api_advanced_settings_form_submit');
  return system_settings_form($form);
}

/**
 * CDEK API advanced settings form validate callback.
 *
 * @param array $form
 *   Drupal form array.
 * @param array $form_state
 *   Drupal form state array.
 *
 * @see cdek_api_advanced_settings_form()
 */
function cdek_api_advanced_settings_form_validate($form, &$form_state) {
  // Get form values.
  $values = &$form_state['values'];
  // Validate request timeout.
  if (!empty($values['cdek_api_request_timeout']) && (!is_numeric($values['cdek_api_request_timeout']) || $values['cdek_api_request_timeout'] < 1)) {
    form_set_error('cdek_api_request_timeout', t('Request timeout is incorrect.'));
  }
  // Validate cache expire.
  $cache_expire = &$values['cdek_api_cache_expire'];
  $cache_expire = trim($cache_expire);
  if ($cache_expire !== '') {
    if (is_numeric($cache_expire) && $cache_expire == round($cache_expire)) {
      $cache_expire = (int) $cache_expire;
      if ($cache_expire <= 0 && $cache_expire != CACHE_PERMANENT && $cache_expire != CACHE_TEMPORARY) {
        form_set_error('cdek_api_cache_expire', t('Cache expire value is incorrect.'));
      }
    }
    else {
      form_set_error('cdek_api_cache_expire', t('Cache expire value is incorrect.'));
    }
  }
}

/**
 * CDEK API advanced settings form submit callback.
 *
 * @param array $form
 *   Drupal form array.
 * @param array $form_state
 *   Drupal form state array.
 *
 * @see cdek_api_advanced_settings_form()
 */
function cdek_api_advanced_settings_form_submit($form, &$form_state) {
  if ($form_state['values']['cdek_api_cache_expire'] === '') {
    // Clear API data cache.
    cache_clear_all('cdek_api', 'cache', TRUE);
  }
}
