<?php

/**
 * @file
 * Contains Cdek class.
 */

/**
 * Wrapper class for CDEK API.
 *
 * See CDEK API documentation for more info.
 */
class Cdek {

  /**
   * Integration API base url.
   *
   * @var string
   */
  protected $integrationApiBasePath;

  /**
   * API base url.
   *
   * @var string
   */
  protected $apiBasePath;

  /**
   * API auth login.
   *
   * @var string
   */
  protected $authLogin;

  /**
   * API secure password.
   *
   * @var string
   */
  protected $securePassword;

  /**
   * API data cache expire.
   *
   * @var int
   */
  protected $cacheExpire;

  /**
   * Cdek class constructor.
   */
  public function __construct() {
    $this->integrationApiBasePath = variable_get('cdek_integration_api_base_path', 'https://integration.cdek.ru');
    $this->apiBasePath = variable_get('cdek_api_base_path', 'https://integration.cdek.ru');
    $this->authLogin = variable_get('cdek_api_auth_login', NULL);
    $this->securePassword = variable_get('cdek_api_secure_password', NULL);
    $cache_expire = variable_get('cdek_api_cache_expire', NULL);
    $this->cacheExpire = is_numeric($cache_expire) ? $cache_expire : NULL;
  }

  /**
   * Load order pickup points.
   *
   * @param array $params
   *   API function params for filtered results.
   *   The following parameters are allowed:
   *   - cityid - City code in CDEK system.
   *   - citypostcode - City postcode string, cityid is more priority.
   *   - type - Pickup point type. Allowed values: "PVZ", "POSTOMAT", "ALL".
   *     Default value is "PVZ".
   *   - countryid - Country code in CDEK system.
   *   - regionid - Region code in CDEK system.
   *   - havecashless - (boolean) Is there a payment terminal.
   *   - isdressingroom - (boolean) Is there a fitting room in the pickup point.
   *   - weightmax - The maximum weight that can be sent to the pickup point.
   *   - allowedcod - (boolean) CASH is allowed in the pickup point.
   *
   * @return array|null
   *   Pickup points array (each point is a stdClass object) or NULL on failure.
   */
  public function getOrderPickupPointsList($params = array()) {
    static $pickup_points_static = NULL;
    if (empty($params)) {
      // Check static value.
      if (isset($pickup_points_static)) {
        return $pickup_points_static;
      }
      // Check cache value.
      if ($this->isCached() && ($cache = cache_get('cdek_api_order_pickup_points')) && isset($cache->data)) {
        return $pickup_points_static = $cache->data;
      }
    }

    // Initialize request options.
    $function_path = 'pvzlist.php';
    $url = url($this->integrationApiBasePath . '/' . $function_path, array(
      'absolute' => TRUE,
      'query' => $params,
    ));
    $method = 'GET';
    $timeout = (float) variable_get('cdek_api_request_timeout', 5);

    // Send API request.
    // TODO: Move API request and XML operations in separate method.
    // TODO: Add logging on failure.
    $response = drupal_http_request($url, array(), $method, NULL, 3, $timeout);
    if ($response->code == '200' && $response->status_message == 'OK') {
      // Try parse xml response.
      try {
        $pickup_points = array();
        $xml = new SimpleXMLElement($response->data);
        foreach ($xml->{'Pvz'} as $xml_element) {
          // Fill pickup point object.
          $pickup_point = new stdClass();
          foreach ($xml_element->attributes() as $name => $value) {
            $pickup_point->{$name} = (string) $value;
          }
          // Set point in points array by code.
          $pickup_points[$pickup_point->Code] = $pickup_point;
        }

        // Set persistent and static cache only for all points request.
        if (empty($params)) {
          $pickup_points_static = $pickup_points;
          // Check cache expire.
          if ($this->isCached()) {
            cache_set('cdek_api_order_pickup_points', $pickup_points, 'cache', $this->cacheExpire);
          }
        }
        return $pickup_points;
      }
      catch (Exception $e) {
        // Can't parse xml string.
        // TODO: Add error logging.
      }
    }
    return NULL;
  }

  /**
   * Load countries.
   *
   * @return array|null
   *   Country names array keyed by country code, or NULL on failure.
   */
  public function getCountries() {
    static $countries = NULL;
    // Check static value.
    if (!isset($countries)) {
      // Check cache value.
      if ($this->isCached() && ($cache = cache_get('cdek_api_countries')) && isset($cache->data)) {
        $countries = $cache->data;
      }
      // Get countries from pickup points.
      elseif ($pickup_points = $this->getOrderPickupPointsList()) {
        $countries = array();
        foreach ($pickup_points as $pickup_point) {
          // Set new country in array.
          if (!isset($countries[$pickup_point->CountryCode])) {
            $countries[$pickup_point->CountryCode] = $pickup_point->CountryName;
          }
        }
        // Sort countries in alphabet.
        asort($countries);
        // Check cache expire.
        if ($this->isCached()) {
          cache_set('cdek_api_countries', $countries, 'cache', $this->cacheExpire);
        }
      }
    }
    return $countries;
  }

  /**
   * Load regions by country.
   *
   * @param int $country
   *   Country code in CDEK system.
   *
   * @return array|null
   *   Region names array keyed by region code, or NULL on failure.
   */
  public function getRegions($country = NULL) {
    static $regions = array();
    // Initialize params for API.
    $params = array();
    empty($country) ? $country = 'All' : $params['countryid'] = $country;

    // Check static value.
    if (!isset($regions[$country])) {
      // Check cache value.
      if ($this->isCached() && ($cache = cache_get("cdek_api_regions:$country")) && isset($cache->data)) {
        $regions[$country] = $cache->data;
      }
      // Get regions from pickup points.
      elseif ($pickup_points = $this->getOrderPickupPointsList($params)) {
        $country_regions = array();
        foreach ($pickup_points as $pickup_point) {
          // Set new region in array.
          if (!isset($country_regions[$pickup_point->RegionCode])) {
            $country_regions[$pickup_point->RegionCode] = $pickup_point->RegionName;
          }
        }
        // Sort regions in alphabet.
        asort($country_regions);
        $regions[$country] = $country_regions;
        // Check cache expire.
        if ($this->isCached()) {
          cache_set("cdek_api_regions:$country", $country_regions, 'cache', $this->cacheExpire);
        }
      }
    }
    return $regions[$country];
  }

  /**
   * Load cities by country and/or region.
   *
   * @param int $country
   *   Country code in CDEK system.
   * @param int $region
   *   Region code in CDEK system.
   *
   * @return array|null
   *   City names array keyed by city code, or NULL on failure.
   */
  public function getCities($country = NULL, $region = NULL) {
    static $cities = array();
    // Initialize params for API.
    $params = array();
    empty($country) ? $country = 'All' : $params['countryid'] = $country;
    empty($region) ? $region = 'All' : $params['regionid'] = $region;

    // Check static value.
    if (!isset($cities[$country][$region])) {
      // Check cache value.
      if ($this->isCached() && ($cache = cache_get("cdek_api_cities:$country:$region")) && isset($cache->data)) {
        $cities[$country][$region] = $cache->data;
      }
      // Get cities from pickup points.
      elseif ($pickup_points = $this->getOrderPickupPointsList($params)) {
        $country_region_cities = array();
        foreach ($pickup_points as $pickup_point) {
          // Set new city in array.
          if (!isset($country_region_cities[$pickup_point->CityCode])) {
            $country_region_cities[$pickup_point->CityCode] = $pickup_point->City;
          }
        }
        // Sort cities in alphabet.
        asort($country_region_cities);
        $cities[$country][$region] = $country_region_cities;
        // Check cache expire.
        if ($this->isCached()) {
          cache_set("cdek_api_cities:$country:$region", $country_region_cities, 'cache', $this->cacheExpire);
        }
      }
    }
    return $cities[$country][$region];
  }

  /**
   * Calculate delivery rate by params.
   *
   * @param array $params
   *   API function params to calculate.
   *   The following parameters are allowed:
   *   - use_auth - (boolean) Request authorization flag, default value is TRUE.
   *   - data_execute - Planning date of shipment in the format YYYY-ММ-DD.
   *   - sender_city_id - Origin city code in CDEK system.
   *   - sender_city_post_code - Zip code of the city of origin.
   *   - receiver_city_id - Destination city code in CDEK system.
   *   - receiver_city_post_code - Zip code of the destination city.
   *   - tariff_id - Code of chosen tariff.
   *   - mode_id - Chosen delivery mode.
   *   - goods - Array with elements each of which contains an
   *     associative array with keys:
   *       - weight - float, piece weight, kg.
   *       - length - integer, piece length, sm.
   *       - width - integer, piece width, sm.
   *       - height - integer, depth, sm.
   *     Or with such kind of parameters:
   *       - weight - float, piece weight, kg.
   *       - volume - float, piece volume, cum.
   *
   * @return object|null
   *   Delivery rate information object (stdClass), or NULL on failure.
   *
   * @throws \Exception
   *   CDEK API exception.
   */
  public function calculateDeliveryRate($params = array()) {
    // Initialize request options.
    $function_path = 'calculator/calculate_price_by_json.php';
    $url = url($this->apiBasePath . '/' . $function_path, array('absolute' => TRUE));
    $headers = array('Content-Type' => 'application/json');
    $method = 'POST';
    $timeout = (float) variable_get('cdek_api_request_timeout', 5);

    // Prepare request entry.
    $data_execute = isset($params['data_execute']) ? $params['data_execute'] : date('Y-m-d', time());
    $values = new stdClass();
    // TODO: Store API version in configuration.
    $values->version = '1.0';
    // Use authorization if necessary.
    if (!isset($params['use_auth']) || (bool) $params['use_auth']) {
      $values->authLogin = $this->authLogin;
      $values->secure = md5($data_execute . '&' . $this->securePassword);
    }
    $values->dateExecute = $data_execute;
    $values->senderCityId = isset($params['sender_city_id']) ? $params['sender_city_id'] : variable_get('cdek_api_sender_city_id', NULL);
    $values->senderCityPostCode = isset($params['sender_city_post_code']) ? $params['sender_city_post_code'] : variable_get('cdek_api_sender_city_postcode', NULL);
    $values->receiverCityId = isset($params['receiver_city_id']) ? $params['receiver_city_id'] : NULL;
    $values->receiverCityPostCode = isset($params['receiver_city_post_code']) ? $params['receiver_city_post_code'] : NULL;
    $values->tariffId = isset($params['tariff_id']) ? $params['tariff_id'] : variable_get('cdek_api_tariff_id', NULL);
    // TODO: tariffList param support.
    $values->modeId = isset($params['mode_id']) ? $params['mode_id'] : NULL;
    $values->goods = isset($params['goods']) ? $params['goods'] : array();

    // Unset empty properties.
    foreach ($values as $key => $value) {
      if (empty($value)) {
        unset($values->{$key});
      }
    }

    // TODO: Move API request in separate method.
    // TODO: Add logging on failure.
    $response = drupal_http_request($url, $headers, $method, json_encode($values), 3, $timeout);
    if ($response->code == 200 && $response->status_message == 'OK') {
      $data = json_decode($response->data);
      // Check API errors.
      if (isset($data->error) && $first_error = array_shift($data->error)) {
        // Throw exception by first error.
        throw new Exception($first_error->text, $first_error->code);
      }
      // Return rate info.
      if (isset($data->result)) {
        return (object) $data->result;
      }
    }
    return NULL;
  }

  /**
   * Verifying that the data is cached.
   *
   * @return bool
   *   API data is cached (TRUE) or not (FALSE).
   */
  protected function isCached() {
    return !is_null($this->cacheExpire);
  }

}
